/*
        Returns to STDOUT:
        varid
        typeid
        type
        type length
        number of dimensions
        dimension ids
        dimension lengths
        dimension names
        number of attributes        
       
        Remik dot Ziemlinski at gfdl dot noaa dott gov
        5.10.2004        
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "netcdf.h" 

#define USAGE "\
Usage: ncinqvar file variable\n\
\n\
Output:\n\
\tvar id\n\
\ttype id\n\
\ttype\n\
\ttype length\n\
\t# dimensions\n\
\tdimension ids\n\
\tdimension lengths\n\
\tdimension names\n\
\t# attributes\n\
\n"

/* Implementation located below */
void handle_error(int status);
void printtype(int type);
void printtypelength(int type);

int main(int argc, char** argv)
{       
        int ncid, ndims, status, i, natts, varid;
        size_t size;
        int dimids[NC_MAX_VAR_DIMS];        
        char name[NC_MAX_NAME];
        nc_type type;     
 














        /********************************************************
        ********************************************************
        ********************************************************/
        
        if (argc <= 2)
        {
                fprintf(stderr, "%s", USAGE);
                return 1;
        }
       
        /* Open the file */
        status = nc_open(argv[1], NC_NOWRITE, &ncid);
        if (status != NC_NOERR) handle_error(status);

        /* Query the variable */
        status = nc_inq_varid(ncid, argv[2], &varid);
        if (status != NC_NOERR) handle_error(status);      

        status = nc_inq_var(ncid, varid, name, &type,
                            &ndims, dimids, &natts);
        if (status != NC_NOERR) handle_error(status);      

        printf("%d\n", varid);
        printf("%d\n", type);
        printtype(type);
        printtypelength(type);
        printf("%d\n", ndims);
        
        for(i=0; i < ndims; ++i)
        {
                printf("%d ", dimids[i]);
        }
        printf("\n");
        
        for(i=0; i < ndims; ++i)
        {
                status = nc_inq_dimlen(ncid, dimids[i], &size);
                if (status != NC_NOERR) handle_error(status);      
                
                printf("%lu ", (unsigned long)size);
        }
        printf("\n");

        for(i=0; i < ndims; ++i)
        {
                status = nc_inq_dimname(ncid, dimids[i], name);
                if (status != NC_NOERR) handle_error(status);      
                
                printf("%s ", name);
        }
        printf("\n");

        printf("%d\n", natts);

        /* Close the file */
        ncclose(ncid);      

        return 0;
}

void handle_error(int status) {

        if (status != NC_NOERR) {        
           fprintf(stderr, "%s\n", nc_strerror(status));
           exit(1);
           }
}
void printtype(int type)
{
        switch(type)
        {
                case NC_BYTE:
                printf("BYTE\n");
                break;
                case NC_CHAR:
                printf("CHAR\n");
                break;
                case NC_SHORT:
                printf("SHORT\n");
                break;
                case NC_INT:
                printf("INT\n");
                break;
                case NC_FLOAT:
                printf("FLOAT\n");
                break;
                case NC_DOUBLE:
                printf("DOUBLE\n");
                break;
        }
}
void printtypelength(int type)
{
        switch(type)
        {
                case NC_BYTE:
                printf("1\n");
                break;
                case NC_CHAR:
                printf("1\n");
                break;
                case NC_SHORT:
                printf("2\n");
                break;
                case NC_INT:
                printf("4\n");
                break;
                case NC_FLOAT:
                printf("4\n");
                break;
                case NC_DOUBLE:
                printf("8\n");
                break;
        }
}
/*======================================================================
  U.S. Department of Commerce (DOC) Software License for program 
  developed at the Geophysical Fluid Dynamics Laboratory/NOAA.

  1. Scope of License

   Subject to all the terms and conditions of this license, DOC grants
   USER the royalty-free, nonexclusive, nontransferable, and worldwide
   rights to reproduce, modify, and distribute this script
   developed at the Geophysical Fluid Dynamics Laboratory/NOAA, herein
   referred to as the Product.

  2. Conditions and Limitations of Use

   Warranties.  Neither the U.S. Government, nor any agency or
     employee thereof, makes any warranties, expressed or implied,
     with respect to the Product provided under this License,
     including but not limited to the implied warranties or
     merchantability and fitness for any particular purpose.

   Liability.  In no event shall the U.S. Government, nor any agency
     or employee thereof, be liable for any  direct, indirect, or
     consequential damages flowing from the use of the Product
     provided under this License.

   Non-Assignment.  Neither this License nor any rights granted
     hereunder are transferable or assignable without the explicit
     prior written consent of DOC. 

   Names and Logos.  USER shall not substitute its name or logo for the
     name or logo of DOC, or any of its agencies, in identification of
     the Product.

   Export of technology.  USER shall comply with all U.S. laws and
     regulations restricting the export of the Product to other
     countries.  

   Governing Law.  This License shall be governed by the laws of 
     United States as interpreted and applied by the Federal courts 
     in the District of Columbia.

  3. Term of License This License shall remain in effect as long as USER
     uses the Product in accordance with Paragraphs 1 and 2.
======================================================================*/
