/*
       Returns a comma separated list of dimension variables found in
       a NetCDF file.
       
        Remik dot Ziemlinski at gfdl dot noaa dott gov
        5.04.2004        
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "netcdf.h" 

/* Implementation located below */
void handle_error(int status);
int isdimension(char** dimnames, char* name, int ndims);

int main(int argc, char** argv)
{       
        /* Read test variables */
        int ncid, ndims, status, i, nvars, natts, varid;
        size_t size;
        int numdims;
        int dimids[NC_MAX_VAR_DIMS];        
        char name[NC_MAX_NAME];
        char* dimnames[NC_MAX_VAR_DIMS];
        nc_type type; 
        int ctr;    
 














        /********************************************************
        ********************************************************
        ********************************************************/
        
        if (argc == 1)
        {
                fprintf(stderr, "Couldn't get variable names. No file specified.\n");
                return 1;
        }
        ctr = 0;
        
        /* allocate memory for names */
        for(i = 0; i < NC_MAX_VAR_DIMS; ++i)
                dimnames[i] = (char*) malloc(sizeof(char)*NC_MAX_NAME);
        
        /* Open the file */
        status = nc_open(argv[1], NC_NOWRITE, &ncid);
        if (status != NC_NOERR) handle_error(status);

        /* Query the number of dimensions */
        status = nc_inq_ndims (ncid, &numdims);
        if (status != NC_NOERR) handle_error(status);
       
        for(i=0; i < numdims; ++i)
        {
                /* Query the dimensions */
                status = nc_inq_dim (ncid, i, name, &size);
                if (status != NC_NOERR) handle_error(status);

                strcpy(dimnames[i], name);                
        }

        /* Query the number of variables */
        status = nc_inq_nvars (ncid, &nvars);
        if (status != NC_NOERR) handle_error(status);      

        if (numdims == 0)
        {
                ctr = 0;
        }        
        if (numdims == 1)
        {
                varid = 0;
                status = nc_inq_var(ncid, varid, name, &type,
                                      &ndims, dimids, &natts);
                if (status != NC_NOERR) handle_error(status);

                if (isdimension(dimnames, name, numdims) == 0)
                        ++ctr;                 
        } else
        {        
                /* loop over all but the last */
                for(i=0; i < nvars; ++i)
                {
                        /* Query the variable */
                        varid = i;
                        status = nc_inq_var(ncid, varid, name, &type,
                                              &ndims, dimids, &natts);
                        if (status != NC_NOERR) handle_error(status);

                        if (isdimension(dimnames, name, numdims) == 0)
                               ++ctr; 
                }            
        }
        
        /* Close the file */
        ncclose(ncid);      

        for(i = 0; i < NC_MAX_VAR_DIMS; ++i)
                free(dimnames[i]);

        printf("%d\n", ctr);
        
        return 0;
}

void handle_error(int status) {

        if (status != NC_NOERR) {        
           fprintf(stderr, "%s\n", nc_strerror(status));
           exit(1);
           }
}
/* Returns 0 if dimension name. */
int isdimension(char** dimnames, char* name, int ndims)
{
        int i;
        
        if (ndims < 1)
                return -1;
        
        for(i = 0; i < ndims; ++i)
        {
        
                if (strcmp(dimnames[i], name) == 0)
                {
                        return 0;
                }
        }
        
        return -1;
}
/*======================================================================
  U.S. Department of Commerce (DOC) Software License for script 
  developed at the Geophysical Fluid Dynamics Laboratory/NOAA.

  1. Scope of License

   Subject to all the terms and conditions of this license, DOC grants
   USER the royalty-free, nonexclusive, nontransferable, and worldwide
   rights to reproduce, modify, and distribute this script
   developed at the Geophysical Fluid Dynamics Laboratory/NOAA, herein
   referred to as the Product.

  2. Conditions and Limitations of Use

   Warranties.  Neither the U.S. Government, nor any agency or
     employee thereof, makes any warranties, expressed or implied,
     with respect to the Product provided under this License,
     including but not limited to the implied warranties or
     merchantability and fitness for any particular purpose.

   Liability.  In no event shall the U.S. Government, nor any agency
     or employee thereof, be liable for any  direct, indirect, or
     consequential damages flowing from the use of the Product
     provided under this License.

   Non-Assignment.  Neither this License nor any rights granted
     hereunder are transferable or assignable without the explicit
     prior written consent of DOC. 

   Names and Logos.  USER shall not substitute its name or logo for the
     name or logo of DOC, or any of its agencies, in identification of
     the Product.

   Export of technology.  USER shall comply with all U.S. laws and
     regulations restricting the export of the Product to other
     countries.  

   Governing Law.  This License shall be governed by the laws of 
     United States as interpreted and applied by the Federal courts 
     in the District of Columbia.

  3. Term of License This License shall remain in effect as long as USER
     uses the Product in accordance with Paragraphs 1 and 2.
======================================================================*/
